#include "NativeModule.h"
#include <hwrmpowerstatesdkpskeys.h>
#include <e32property.h>

_LIT(BATTERY, "battery");
_LIT(LOW, "low");
_LIT(OK, "ok");
_LIT(EMPTY, "empty");
_LIT(ERROR, "error");

class battery : public NativeModule
	{
private:
	///definisco le funzioni
	enum
		{
		ChargeFunction, PlugStatusFunction, BattStatusFunction, BattLevelFunction
		};
	
	/**
	 *  Ottiene la proprietà richiesta 
	 */
	TInt getProp(TUint32 prop) 
		{
			RProperty rp;
			TInt handle;
			
			TInt retval = rp.Get(KPSUidHWRMPowerState, prop, handle);
			
			if (retval == 0)
				return handle;
			else
				return -1; //errore
		}

	/**
	 *  Ritorna 'true' se il dispositivo collegato � in carica.
	 */
	Runtime::Value getChargingStatus()
		{
		Runtime::Value retval;

		if (getProp(KHWRMChargingStatus) == EChargingStatusCharging
				|| getProp(KHWRMChargingStatus) == EChargingStatusAlmostComplete
				|| getProp(KHWRMChargingStatus) == EChargingStatusChargingContinued)
			retval.SetBoolean(true);
		else
			retval.SetBoolean(false);

		return retval;
		}

	/**
	 * Ritorna lo stato di carica della batteria del telefono 
	 */
	Runtime::Value getBatteryStatus()
		{
		Runtime::Value status;
		status.SetNumber(getProp(KHWRMBatteryStatus));
		return status;
		}
	
	Runtime::Value getLevel()
		{
		Runtime::Value level;
		level.SetNumber(getProp(KHWRMBatteryLevel));
		return level;
		}

	/**
	 *	Ritorna 'true' se il dispositivo è collegato all'alimentazione elettrica. 
	 */
	Runtime::Value getPlugStatus()
		{
		Runtime::Value retval;
		if (getProp(KHWRMChargingStatus) == EChargingStatusNotConnected)
			{
			retval.SetBoolean(false);
			}
		else
			{
			retval.SetBoolean(true);
			}

		return retval;
		}

protected:

	TInt ExpectedRuntimeVersion()
		{
		return Runtime::VERSION;
		}

	void ConstructL()
		{		
		runtime->AddNativeFunctionL(_L("charge"), 0, 0, ChargeFunction);
		runtime->AddNativeFunctionL(_L("plugged"), 0, 0, PlugStatusFunction);
		runtime->AddNativeFunctionL(_L("state"), 0, 0, BattStatusFunction);
		runtime->AddNativeFunctionL(_L("level"), 0, 1, BattLevelFunction);

		runtime->AddConstantL(_L("ok"), EBatteryStatusOk);
		runtime->AddConstantL(_L("low"), EBatteryStatusLow);
		runtime->AddConstantL(_L("empty"), EBatteryStatusEmpty);
		runtime->AddConstantL(_L("error"), -1);
		}

	Runtime::Value ExecuteL(TInt index, Runtime::Value *params,
			TInt paramCount, TRequestStatus &status)
		{
		Runtime::Value retval;
		switch (index)
			{
			case ChargeFunction:
				retval = getChargingStatus();
				break;
			case PlugStatusFunction:
				retval = getPlugStatus();
				break;
			case BattStatusFunction:
				retval = getBatteryStatus();
				break;
			case BattLevelFunction:
				retval = getLevel();
				if(paramCount == 1 && !params[0].IsNull())
					{
					if(params[0].IsBoolean() && params[0].GetBooleanL() == true)
					// 7:100 = level:x --> x = (100*level) / 7 (max level)
						{
						TInt value = retval.GetIntL();
						TInt res = (value * 100) / EBatteryLevelLevel7;
						retval.SetNumber(res);	
						}
					}
				break;
			}
		return retval;
		}
	};

EXPORT_C NativeModule* NewBatteryModuleL()
	{
	return new (ELeave)battery;
	}

#ifndef EKA2
GLDEF_C TInt E32Dll(TDllReason)
	{return KErrNone;}
#endif
